--   This file is part of Reno desktop.
--
--  Reno desktop is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
--
--  Reno desktop is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License along with Reno desktop. If not, see <https://www.gnu.org/licenses/>. 
-- Global context menu base
local awmtk2 = require("awmtk2")
local wibox = require("wibox")
local gears = require("gears")
local awful = require("awful")
local menugen = require("context_menu")
local builder = require("builder")

return function(args)
    local style = awmtk2.create_style("root_menu",
        awmtk2.generic.composite_widget,{})
    local templates = awmtk2.create_template_lib("root_menu",awmtk2.templates,{})
    local t = awmtk2.build_templates(templates,style,args.vertical)
    -- Layout configuration
    local config_file = io.open(root_path.."/themes/"..global.theme.."/config/root_menu.json","r")
    local config
    if config_file then
        config = config_file:read("*a")
        config_file:close()
    else
        config = [[{"list": [{"widget": "widgets.rootcontrols"}],"vertical": true}]]
    end
    -- NOTE: **PLEASE** attach your other menus to this thing, ok? IF YOU DON'T, THE MENUS WILL WORK LIKE SHIT
    -- NOTE 2: THERE IS NO OTHER "PROPER" FIX FOR THIS, JUST LOOK AT THE PREVIOUS SYSTEM (SPOILER: it was ABSOLUTE GARBAGE)
    -- Primary menu to which other menus get attached
    local prime_menu = menugen({items = {}})
    -- Not yet loaded menu popup (TODO: maybe make it more obvious that it's a global?)
    root_menu = awful.popup(t.popup({
        markup = "brainhurt the game",
        prime_menu,
        widget = wibox.widget.textbox
    }))
    -- Close popup on right click
    root_menu:connect_signal("button::press",function(_,_,_,b)
        if b == 3 then
            root_menu.visible = false
        end
    end)
    -- Build the menu based on the json config
    root_menu.widget = wibox.widget(t.popup(builder(
        config,
        {
            style = style.base,
            screen = mouse.screen,
            passthrough = {
                -- This is the menu to which other menus should attach (available as args.menu_parent)
                menu_parent = prime_menu,
            }
        }
    )).widget)
    -- Close the menu if prime_menu received a cascade::kill signal
    prime_menu:connect_signal("cascade::kill",function()
        root_menu.visible = false
    end)
    -- Close the prime_menu if our menu becomes invisible
    root_menu:connect_signal("property::visible",function()
        if not root_menu.visible then
            prime_menu:emit_signal_recursive("cascade::close")
        end
    end)
    -- Make the root_menu pop up on the desktop on right click
    local buttons = root.buttons()
    root.buttons(gears.table.join(buttons,
        awful.button({}, 3, function()
            root_menu.visible = (not root_menu.visible)
            if root_menu.visible then
                root_menu:move_next_to(gears.table.join(mouse.coords(),{
                    width = 0,
                    height = 0
                }))
            end
        end)
    ))
    return root_menu
end
